"use strict";Object.defineProperty(exports, "__esModule", {value: true});var _biggystring = require('biggystring');
var _ethers = require('ethers');


var _uniV2Contracts = require('./uniV2Contracts');
/**
 * Get the output swap amounts based on the requested input amount.
 * Call the router contract to calculate amounts and check if the swap path is
 * supported.
 */
 const getSwapAmounts = async (
  router,
  quoteFor,
  nativeAmount,
  path,
  isWrappingSwap
) => {
  const [amountToSwap, expectedAmountOut] = (isWrappingSwap
    ? [nativeAmount, nativeAmount]
    : quoteFor === 'to'
    ? await router.getAmountsIn(nativeAmount, path)
    : quoteFor === 'from'
    ? await router.getAmountsOut(nativeAmount, path)
    : []
  ).map(String)

  if (amountToSwap == null || expectedAmountOut == null)
    throw new Error(`Failed to calculate amounts`)

  return { amountToSwap, expectedAmountOut }
}; exports.getSwapAmounts = getSwapAmounts

/**
 * Get smart contract transaction(s) necessary to swap based on swap params
 */
 const getSwapTransactions = async (
  provider,
  inOutAddresses,
  path,
  router,
  wrappedTokenContract,
  amountToSwap,
  expectedAmountOut,
  toAddress,
  slippage,
  deadline,
  previousGasPrice
) => {
  const {
    fromTokenAddress,
    isFromNativeCurrency,
    isToNativeCurrency,
    isFromWrappedCurrency,
    isToWrappedCurrency
  } = inOutAddresses

  // Determine router method name and params
  if (isFromNativeCurrency && isToNativeCurrency)
    throw new Error('Invalid swap: Cannot swap to the same native currency')

  const gasPrice =
    previousGasPrice != null
      ? _ethers.ethers.utils.parseUnits(previousGasPrice, 'gwei')
      : await provider.getGasPrice()

  const addressToApproveTx = async (
    tokenAddress,
    contractAddress
  ) => {
    const tokenContract = _uniV2Contracts.makeErc20Contract.call(void 0, provider, tokenAddress)
    const promise = tokenContract.populateTransaction.approve(
      contractAddress,
      _ethers.BigNumber.from(amountToSwap),
      { gasLimit: '60000', gasPrice }
    )
    return await promise
  }

  const txPromises = []

  // Deposit native currency for wrapped token
  if (isFromNativeCurrency && isToWrappedCurrency) {
    txPromises.push(
      ...[
        wrappedTokenContract.populateTransaction.deposit({
          gasLimit: '60000',
          gasPrice,
          value: amountToSwap
        })
      ]
    )
  }
  // Withdraw wrapped token for native currency
  else if (isFromWrappedCurrency && isToNativeCurrency) {
    txPromises.push(
      // Deposit Tx
      wrappedTokenContract.populateTransaction.withdraw(amountToSwap, {
        gasLimit: '60000',
        gasPrice
      })
    )
  }
  // Swap native currency for token
  else {
    const slippageMultiplier = _biggystring.sub.call(void 0, '1', slippage)
    if (isFromNativeCurrency && !isToNativeCurrency) {
      txPromises.push(
        // Swap Tx
        router.populateTransaction.swapExactETHForTokens(
          _biggystring.round.call(void 0, _biggystring.mul.call(void 0, expectedAmountOut, slippageMultiplier), 0),
          path,
          toAddress,
          deadline,
          { gasLimit: '250000', gasPrice, value: amountToSwap }
        )
      )
    }
    // Swap token for native currency
    else if (!isFromNativeCurrency && isToNativeCurrency) {
      txPromises.push(
        // Approve TX
        addressToApproveTx(fromTokenAddress, router.address),
        // Swap Tx
        router.populateTransaction.swapExactTokensForETH(
          amountToSwap,
          _biggystring.round.call(void 0, _biggystring.mul.call(void 0, expectedAmountOut, slippageMultiplier), 0),
          path,
          toAddress,
          deadline,
          { gasLimit: '250000', gasPrice }
        )
      )
    }
    // Swap token for token
    else if (!isFromNativeCurrency && !isToNativeCurrency) {
      txPromises.push(
        // Approve TX
        addressToApproveTx(fromTokenAddress, router.address),
        // Swap Tx
        router.populateTransaction.swapExactTokensForTokens(
          amountToSwap,
          _biggystring.round.call(void 0, _biggystring.mul.call(void 0, expectedAmountOut, slippageMultiplier), 0),
          path,
          toAddress,
          deadline,
          { gasLimit: '600000', gasPrice }
        )
      )
    } else {
      throw new Error('Unhandled swap type')
    }
  }

  return await Promise.all(txPromises)
}; exports.getSwapTransactions = getSwapTransactions
